searchData={"items":[{"type":"module","title":"PintBroker","doc":"A simple, pint-sized MQTT broker that can be used for testing and development\n\nSee [the README](https://hexdocs.pm/pint_broker/readme.html) for more information\non usage, features, and configuration","ref":"PintBroker.html"},{"type":"function","title":"PintBroker.add_rule/3","doc":"Add a topic filter rule with handler\n\nSee [Rule forwarding](https://hexdocs.pm/pint_broker/readme.html#rule-forwarding) for more info","ref":"PintBroker.html#add_rule/3"},{"type":"function","title":"PintBroker.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"PintBroker.html#child_spec/1"},{"type":"function","title":"PintBroker.publish/3","doc":"Publish a message to a topic.","ref":"PintBroker.html#publish/3"},{"type":"function","title":"PintBroker.start_link/1","doc":"Start and link a PintBroker\n\nSee `PintBroker.opt()` and [Configuration](https://hexdocs.pm/pint_broker/readme.html#configuration)\nfor more info about configuring the broker","ref":"PintBroker.html#start_link/1"},{"type":"type","title":"PintBroker.opt/0","doc":"","ref":"PintBroker.html#t:opt/0"},{"type":"type","title":"PintBroker.rule/0","doc":"","ref":"PintBroker.html#t:rule/0"},{"type":"extras","title":"PintBroker 🍺","doc":"# PintBroker 🍺\n\n[![CircleCI](https://circleci.com/gh/jjcarstens/pint_broker.svg?style=svg)](https://circleci.com/gh/jjcarstens/pint_broker)\n[![Hex version](https://img.shields.io/hexpm/v/pint_broker.svg \"Hex version\")](https://hex.pm/packages/pint_broker)\n\nA simple, pint-sized MQTT broker that can be used for testing and development\n\n> [!WARNING]\n> This is not indended for production use and makes no attempts for large\n> connection scaling and handling. It is also not considered feature complete,\n> but handles most simple use cases.\n\n**Supported:**\n\n* Automatic startup or as part of your supervision tree (see [Usage](#usage))\n* Simple, unencrypted TCP connections\n* [MQTT v3.1.1](http://docs.oasis-open.org/mqtt/mqtt/v3.1.1/os/mqtt-v3.1.1-os.pdf)\n  * QoS 0\n  * Connect, publish, subscribe, and unsubscribe\n  * Ping requests\n* [Rule forwarding](#rule-forwarding)\n* [Lifecycle events](#lifecycle-events)\n\n**Unsupported:**\n\n* SSL connections\n* QoS 1 and 2\n* [MQTT v5](https://docs.oasis-open.org/mqtt/mqtt/v5.0/mqtt-v5.0.pdf)","ref":"readme.html"},{"type":"extras","title":"Usage - PintBroker 🍺","doc":"`PintBroker` can be started in your own supervision tree or automatically\nas an OTP application by including `:pint_broker` in your applications\n`config.exs` file. It is generally recommended to start the broker in your\nsupervision tree for the most control and flexibility.","ref":"readme.html#usage"},{"type":"extras","title":"Starting as part of your supervision tree - PintBroker 🍺","doc":"> [!IMPORTANT]\n> Ensure there is no `:pint_broker` key in your `config.exs` as that\n> is used to determine starting the broker as an OTP application\n\nPass any of the supported [configuration options](#configuration) as\npart of the child spec:\n\n```elixir\n{PintBroker, broker_opts}\n```","ref":"readme.html#starting-as-part-of-your-supervision-tree"},{"type":"extras","title":"Starting as an OTP application - PintBroker 🍺","doc":"If you include any of the supported [configuration options](#configuration)\nin your `config.exs` file, the broker will be started as a dependency of \nyour application and rely on the Application environment for configuration:\n\n```elixir\nconfig :pint_broker, port: 1883, name: CoolBroker\n```","ref":"readme.html#starting-as-an-otp-application"},{"type":"extras","title":"Configuration - PintBroker 🍺","doc":"The following configuration options are supported:\n\n* `:port` - listening port for incoming connections (default: `1883`)\n* `:name` - Valid GenServer name (default: `PintBroker`)\n* `:rules` - list of message forwarding rules. See [Rule forwarding](#rule-forwarding)\n* `:on_connect` - callback function for connect events. See [Lifecycle events](#lifecycle-events)\n* `:on_disconnect` - callback function for disconnect events. See [Lifecycle events](#lifecycle-events)\n* `:overrides` - additional options to pass to `:gen_tcp.listen/2`. These take\n  precedence and are unchecked. Be careful using this since it can break other options.","ref":"readme.html#configuration"},{"type":"extras","title":"Rule forwarding - PintBroker 🍺","doc":"Many production setups will have a few topics with rules that forward\nmessages to a handler such as an SQS queue or central service. This allows\nfor scaling of many devices to communicate with a few nodes. For testing,\nyou can specify rules when starting the broker which will forward\n`Tortoise311.Package.Publish` structs to a handler function or process in\norder to mimic this rule forwarding behavior. See `PintBroker.add_rule/3` for\nmore information.\n\n**Example:**\n\n```elixir\niex> s = self()\n\niex> handler1 = fn pub -> send(s, pub) end\n\niex> PintBroker.start_link(rules: [{\"my/+/test/topic\", handler1}])\n{:ok, #PID<0.226.0>}\n\n# You can publish from the broker or another client\niex> PintBroker.publish(\"my/first/test/topic\", \"hello world\")\n\niex> flush()\n%Tortoise311.Package.Publish{\n  __META__: %Tortoise311.Package.Meta{opcode: 3, flags: 0},\n  identifier: nil,\n  topic: \"my/first/test/topic\",\n  payload: \"hello world\",\n  qos: 0,\n  dup: false,\n  retain: false\n}\n```","ref":"readme.html#rule-forwarding"},{"type":"extras","title":"Lifecycle events - PintBroker 🍺","doc":"Many broker setups have mechanism to subscribe to connect/disconnect events,\nsuch as the [AWS IoT Lifecycle events](https://docs.aws.amazon.com/iot/latest/developerguide/life-cycle-events.html)\nwhich are typically configured in the infrastructure to be forwarded to\na central handler or SQS queue.\n\nPintBroker also supports this behavior by starting with `:on_connect` and\n`:on_disconnect` options to register a callback function that receives\nthe `:client_id` of the connection.\n\nAn example for replicating the AWS IoT Lifecyle events:\n\n```elixir\non_connect = fn client_id ->\n  payload = %{clientId: client_id, eventType: :connected}\n  Broadway.test_message(:my_broadway, Jason.encode!(payload))\nend\n\non_disconnect = fn client_id ->\n  payload = %{clientId: client_id, eventType: :disconnected, disconnectReason: \"CONNECTION_LOST\"}\n  Broadway.test_message(:my_broadway, Jason.encode!(payload))\nend\n\nPintBroker.start_link(on_connect: on_connect, on_disconnect: on_disconnect)\n```\n\nThe callback returns are ignored.","ref":"readme.html#lifecycle-events"},{"type":"extras","title":"Why _another_ broker? - PintBroker 🍺","doc":"There are many full-featured MQTT brokers out there, but they require a lot of\nsetup and configuration, maybe some dependencies, and tend to be overkill for\ntesting of MQTT interactions between client and servers. I wanted the simplest\npossible broker which did not require a full ops team to implement a test\nenvironment that clients could connect to and publish/subscribe to topics for\nimproved MQTT unit testing and local development of the whole system.","ref":"readme.html#why-_another_-broker"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.1.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).","ref":"changelog.html"},{"type":"extras","title":"v1.0.2 - 2024-06-23 - Changelog","doc":"","ref":"changelog.html#v1-0-2-2024-06-23"},{"type":"extras","title":"Fixed - Changelog","doc":"* Remove warning when `name` option is used for GenServer naming","ref":"changelog.html#fixed"},{"type":"extras","title":"v1.0.1 - 2023-11-18 - Changelog","doc":"","ref":"changelog.html#v1-0-1-2023-11-18"},{"type":"extras","title":"Added - Changelog","doc":"* Support starting as an OTP application or in supervision\n* Callbacks to support lifecycle events. See [Lifecycle events](https://hexdocs.pm/pint_broker/readme.html#lifecycle-events) for more info","ref":"changelog.html#added"},{"type":"extras","title":"v1.0.0 - 2023-09-26 - Changelog","doc":"Initial release. See [README.md](README.md) for more information.","ref":"changelog.html#v1-0-0-2023-09-26"},{"type":"extras","title":"Added - Changelog","doc":"* Simple, unencrypted TCP connections\n* [MQTT v3.1.1](http://docs.oasis-open.org/mqtt/mqtt/v3.1.1/os/mqtt-v3.1.1-os.pdf)\n  * QoS 0\n  * Connect, publish, subscribe, and unsubscribe\n  * Ping requests\n  * Rule forwarding (see below)","ref":"changelog.html#added"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,49]}}